from bn import absimport

default_help_string = """\
usage: ``%(program)s COMMAND [OPTIONS] ARGS``

Commands (aliases):

 :serve:    start the server
 :instance: set up or destroy an instance

Try \`%(program)s COMMAND --help' for help on a specific command.
"""

default_option_sets = {
    'port': [
        dict(
            type = 'command',
            long = ['--port'],
            short = [],
            metavar = 'PORT'
        ),
    ],
    'host': [
        dict(
            type = 'command',
            long = ['--host'],
            short = [],
            metavar = 'HOST',
        ),
    ],
    'profile': [
        dict(
            type = 'command',
            long = ['--profile'],
            short = [],
        ),
    ],
    'help': [
        dict(
            type = 'shared',
            long = ['--help'],
            short = ['-h'],
        ),
    ],
    'verbose': [
        dict(
            type = 'program',
            long = ['--verbose'],
            short = ['-v'],
        ),
        dict(
            type = 'program',
            long = ['--quiet'],
            short = ['-q'],
        ),
    ],
}

def get_default_command_handler_factories(flow):

    def make_handle_command_serve():
        return absimport(
            flow.config.app.package+'.cmd.serve'
        ).handle_command_serve

    def make_handle_command_instance():
        return absimport(
            flow.config.app.package+'.cmd.instance'
        ).handle_command_instance

    def make_handle_command_shell():
        return absimport(
            flow.config.app.package+'.cmd.shell'
        ).handle_command_shell

    command_handler_factories = {
        'serve': make_handle_command_serve,
        'instance': make_handle_command_instance,
        'shell': make_handle_command_shell,
    }
    return command_handler_factories

def get_default_help_string(flow):
    return default_help_string

def get_default_option_sets(flow):
    return default_option_sets

