import getopt
import os

from configconvert import parse_config
from commandtool import set_error_on
from bn import absimport, AttributeDict
from flows.provider import start_flow, ServiceProvider
from flows.framework.config import set_up_app_options

def handle_command_instance(
    flow
):
    """\
    usage: ``%(program)s instance MODE CONFIG_FILE``

    Serve the application based on settings in CONFIG_FILE

    Arguments:
      :``MODE``:        Can be create drop or recreate
      :``CONFIG_FILE``: The full path (including filename) to the config file
                        to use

    For PROGRAM_OPTIONS type \`%(program)s --help'
    """
    option_sets = flow.cmd.option_sets
    command_options = flow.cmd.command_options
    args = flow.cmd.args
    config = flow.config
    options = config.option

    if not len(args)>=1:
        raise getopt.GetoptError('No mode specified')
    mode = args[0]

    # Get the available modes:
    mode_handlers = absimport(
        config.app.package+'.framework.instance'
    ).on_load_mode_handlers(flow)

    if mode not in mode_handlers.keys():
        raise getopt.GetoptError("Unknown MODE %r"%mode)

    framework_services = absimport(
        config.app.package+'.framework.service'
    ).on_load_services(flow)
    dynamic_services = absimport(
        config.app.package+'.dynamic.service'
    ).on_load_services(flow)
    available_services = {}
    available_services.update(dynamic_services)
    # framework services over-write dynamic ones
    available_services.update(framework_services)

    start_flow(
        mode_handlers[mode],
        ServiceProvider(
            existing_services=flow,
            available_services=available_services,
        ),
    )

