import getopt
import os

from commandtool import set_error_on
from bn import absimport, uniform_path, AttributeDict

def handle_command_serve(
    flow,
):
    """\
    usage: ``%(program)s serve [OPTIONS]``

    Serve the application based on settings in the config file 

    Options:
     server.host=HOST     set the host option
     server.port=PORT     set the port option
     server.profile=True  run the profiler too

    For PROGRAM_OPTIONS type \`%(program)s --help'
    """
    option_sets = flow.cmd.option_sets
    command_options = flow.cmd.command_options
    args = flow.cmd.args
    config = flow.config
    options = config.option
    internal_vars = set_error_on(
        command_options,
        allowed=['port', 'host', 'profile']
    )
    # Imports
    on_load_application = absimport(
        options['app.package']+'.framework.application', 
    ).on_load_application

    flow = AttributeDict(dict(config=config))
    
    app = on_load_application(flow)

    if config.server.profile:
        # sudo apt-get install python-profiler
        from repoze.profile.profiler import AccumulatingProfileMiddleware
        app = AccumulatingProfileMiddleware(
            app,
            log_filename='profile.log',
            cachegrind_filename='profile.cachegrind',
            discard_first_request=True,
            flush_at_shutdown=True,
            path='/__profile__'
        )

    # Now use the application to call the serve() function 
    serve(
        app = app,
        host = config.server.host,
        port = config.server.port,
    )

def serve(app, host='', port=8000, profile=False):
    """Start a server with the ``app``, ``host`` and ``port`` specified."""
    try:
        from paste.httpserver import serve
    except:
        from wsgiref.simple_server import make_server
        httpd = make_server(host, port, app)
        print "Serving HTTP with wsgiref on http://%s:%s/"%(host, port)
        httpd.serve_forever()
    else:
        print "Serving HTTP with Paste"
        serve(app, host, port)


