import getopt
import os
import code
import sys

from commandtool import set_error_on
from bn import absimport
from flows.provider import start_flow, ServiceProvider

class FlowsExitError(Exception):
    pass

class FlowsInteractiveConsole(code.InteractiveConsole):

    cancelled = False

    def runcode(self, code_):
        """Execute a code object.

        When an exception occurs, self.showtraceback() is called to
        display a traceback.  All exceptions are caught except
        SystemExit, which is reraised.

        A note about KeyboardInterrupt: this exception may occur
        elsewhere in this code, and may not always be caught.  The
        caller should be prepared to deal with it.

        """
        try:
            exec code_ in self.locals
        except SystemExit:
            raise FlowsExitError()
        except FlowsExitError:
            self.cancelled = True
            raise
        except:
            self.showtraceback()
        else:
            if code.softspace(sys.stdout, 0):
                print

def handle_command_shell(
    flow,
):
    """\
    usage: ``%(program)s shell``

    Load a debug shell with a flow object present for interactive changes
    """
    available_services = absimport(
        flow.config.app.package+'.framework.service'
    ).on_load_services(flow)
    available_services.update(
        absimport(
            flow.config.app.package+'.dynamic.service',
        ).on_load_services(flow)
    )

    def run(flow):
        def cancel():
            raise FlowsExitError()
        c = FlowsInteractiveConsole(
            {
                'flow': flow,
                'cancel': cancel,
            }
        )
        try:
            c.interact(
                'Flows interactive console providing an initialised flow object\n'
                'Type exit() or Ctrl+D to exit and stop services normally\nType '
                'cancel() to exit triggering error handling'
            )
        except FlowsExitError:
            if c.cancelled:
                flow.provider['error_handled'] = True
                raise Exception('Triggering error handling')
    start_flow(
        run, 
        provider = ServiceProvider(
            existing_services=flow,
            available_services=available_services,
        ),
    )


