import logging
import os

from bn import absimport 
from flows.framework.application import not_found

log = logging.getLogger(__name__)

def make_dynamic_component(flow, required_services):
    available_services = absimport(
        flow.config.app.package+'.dynamic.service',
    ).on_load_services(flow)
    if not hasattr(flow, 'url') and not 'url' in required_services:
        flow.provider.register(available_services['url'])
    flow.provider.register(available_services)

    def component(flow):
        mod = None
        action = None
        for new_service in required_services:
            flow.provider.start(new_service, False)
        vars = flow.url.vars
        if vars is None:
            log.error(
                'Could not route %r to an action, no URL vars found',
                flow.http.environ['PATH_INFO'],
            )
            return not_found(flow)
        elif not vars:
            log.error(
                'Could not route %r to an action, no URL vars matched',
                flow.http.environ['PATH_INFO'],
            )
            return not_found(flow)
        if not vars.get('controller'):
            log.error(
                'Could not route %r to an action, no controller specified',
                flow.http.environ['PATH_INFO'],
            )
            return not_found(flow)
        controller_name = vars.get('controller')
        if not os.path.exists(
            os.path.join(
                flow.config.app.app_dir, 
                'dynamic',
                'area',  
                controller_name, 
                'action.py'
            )
        ):
            log.error(
                'The actions.py file for controller %r matched from %r '
                'does not exist ',
                flow.http.environ['PATH_INFO'],
            )
            return not_found(flow)
        if not vars.get('action'):
            log.error(
                'Could not route %r to an action, no action specified by '
                'matched rule',
                flow.http.environ['PATH_INFO'],
            )
            return not_found(flow)
        action_name = 'on_action_'+vars['action']
        module = absimport(
            flow.config.app.package+'.dynamic.area.%s.action'%controller_name,
        )
        if not hasattr(module, action_name):
            log.error(
                'No such action %r in controller %r matched from %r ',
                action_name,
                controller_name,
                flow.http.environ['PATH_INFO'],
            )
            return not_found(flow)

        action = getattr(module, action_name)
        result = action(flow)
        if result:
            flow.http.response.body.append(result)
    return component
