import getopt
import logging
import os
import sys
import warnings

from bn import absimport, AttributeDict
from commandtool import parse_command_line
from commandtool import handle_program
from configconvert import parse_config
from flows.framework.config import create_config

log = logging.getLogger(__name__)

def run():
    program_name = 'python -m flows.run %s'%(sys.argv[1])
    command = None
    try:
        sys_args = sys.argv[:]
        config_file = sys_args.pop(1)
        config = create_config(config_file)
        cmd = absimport(
            config.app.package+'.cmd'
        )
        config_flow = AttributeDict(config=config)
        commands = cmd.on_load_commands(config_flow)

        class Help(object):
            __program__ = commands['help_string']

        aliases = dict(
            [cmd, []] for cmd in commands['command_handler_factories'].keys()
        )
        program_options, command_options, command, args = parse_command_line(
            commands['option_sets'],
            aliases,
            sys_args,
        )
        handle_program(
            command_handler_factories=commands['command_handler_factories'],
            option_sets=commands['option_sets'],
            aliases=aliases,
            program_options=program_options,
            command_options=command_options,
            command=command,
            args=args,
            program_name=program_name,
            help=Help(),
            existing=config_flow,
        )
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err)
        if command:
            print "Try `%(program)s %(command)s --help' for more information." % {
                'program': program_name,
                'command': command,
            }
        else:
            print "Try `%(program)s --help' for more information." % {
                'program': program_name,
            }
        sys.exit(2)

if __name__ == '__main__':
    run()

